﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Imaging;
using System.Reflection;

namespace QLFUI
{
    public partial class MainForm : Form
    {

        #region 变量

        bool needMaxbt = true;  //是否需要最大化按钮
        bool needMinbt = true; //是否需要最小化按钮

        //记录原始的用户设置的大小，用于最大化后返回正常的大小
        int orginWidth;
        int orginHeight;

        //记录窗体的位置
        int X;
        int Y;


        //窗体的状态
        enum stat
        {
            Normal, Max, Min
        }
        stat st = stat.Normal;
        #endregion

        #region 属性

        /// <summary>
        /// 隐藏size属性,通过BorderWidth，BorderHeight设置
        /// </summary>
        [Browsable(false)]
        public new SizeF Size
        {
            get { return base.Size; }
        }

        [CategoryAttribute("QLFSkinDll"), Description("设置窗体宽度")]
        public int BorderWidth
        {
            get { return this.Width; }
            set
            {
                if (value < 180)
                {
                    MessageBox.Show("宽度太小了，宽度最少是180个像素。");
                    return;
                }
                if (this.Width != value)
                {
                    int newwidth = value;
                    this.Width = value;

                    ptbml.Left = 1;
                    ptbmr.Left = newwidth - 4;
                    ptbtr.Left = newwidth - 10;
                    ptbbr.Left = newwidth - 10;

                    panelm.Width = newwidth - 7;
                    panelb.Width = newwidth - 20;
                    panelt.Width = newwidth - 20;

                    btclose.Left = newwidth - 57;
                    btmin.Left = newwidth - 120;
                    btmax.Left = newwidth - 89;
                    btres.Left = newwidth - 89;
                }

            }
        }

        [CategoryAttribute("QLFSkinDll"), Description("设置窗体高度")]
        public int BorderHeight
        {
            get { return this.Height; }
            set
            {
                if (this.Height != value)
                {
                    int newheight = value;
                    this.Height = value;

                    panelm.Height = newheight - 68;
                    ptbml.Height = newheight - 68;
                    ptbmr.Height = newheight - 68;
                    ptbml.Top = 31;
                    ptbmr.Top = 31;

                    panelb.Top = newheight - 37;
                    ptbbl.Top = newheight - 37;
                    ptbbr.Top = newheight - 37;
                }

            }
        }

        [CategoryAttribute("QLFSkinDll"), Description("设置窗体标题")]
        public string FormTitle
        {
            get { return lbtitle.Text; }
            set { lbtitle.Text = value; }
        }

        [CategoryAttribute("QLFSkinDll"), Description("是否需要最大化按钮")]
        public bool NeedMax
        {
            get { return needMaxbt; }
            set
            {
                needMaxbt = value;
                if (value == false)
                {
                    this.btmax.Top = -50;
                    this.btmin.Left = this.Width - 89;
                }
                else
                {
                    this.btmax.Top = 0;
                    this.btmin.Left = this.Width - 120;
                }
            }
        }

        [CategoryAttribute("QLFSkinDll"), Description("是否需要最小化按钮")]
        public bool NeedMix
        {
            get { return needMinbt; }
            set
            {
                needMinbt = value;
                if (value == false)
                {
                    this.btmin.Top = -50;
                }
                else
                {
                    this.btmin.Top = 0;
                }
            }
        }

        #endregion

        #region 构造函数

        public MainForm()
        {

            this.SetStyle(ControlStyles.UserPaint, true);
            this.SetStyle(ControlStyles.AllPaintingInWmPaint, true);
            this.SetStyle(ControlStyles.OptimizedDoubleBuffer, true);

            InitializeComponent();

            //左上角
            this.ptbtr.BackgroundImage = new Bitmap(Image.FromStream(Assembly.GetExecutingAssembly().GetManifestResourceStream(@"QLFUI.Res.border.bordertr.bmp")));
            Bitmap b = this.ptbtr.BackgroundImage as Bitmap;
            b.MakeTransparent(Color.FromArgb(255, 0, 255));

            //右上角
            this.ptbtl.BackgroundImage = new Bitmap(Image.FromStream(Assembly.GetExecutingAssembly().GetManifestResourceStream(@"QLFUI.Res.border.bordertl.bmp")));
            Bitmap b1 = this.ptbtl.BackgroundImage as Bitmap;
            b1.MakeTransparent(Color.FromArgb(255, 0, 255));

            //左下角
            this.ptbbl.BackgroundImage = new Bitmap(Image.FromStream(Assembly.GetExecutingAssembly().GetManifestResourceStream(@"QLFUI.Res.border.borderbl.bmp")));
            Bitmap b2 = this.ptbbl.BackgroundImage as Bitmap;
            b2.MakeTransparent(Color.FromArgb(255, 0, 255));

            //右下角
            this.ptbbr.BackgroundImage = new Bitmap(Image.FromStream(Assembly.GetExecutingAssembly().GetManifestResourceStream(@"QLFUI.Res.border.borderbr.bmp")));
            Bitmap b3 = this.ptbbr.BackgroundImage as Bitmap;
            b3.MakeTransparent(Color.FromArgb(255, 0, 255));


            //中间左
            this.ptbml.BackgroundImage = new Bitmap(Image.FromStream(Assembly.GetExecutingAssembly().GetManifestResourceStream(@"QLFUI.Res.border.borderml.bmp")));
            Bitmap b4 = this.ptbtl.BackgroundImage as Bitmap;
            b4.MakeTransparent(Color.FromArgb(255, 0, 255));

            //中间右
            this.ptbmr.BackgroundImage = new Bitmap(Image.FromStream(Assembly.GetExecutingAssembly().GetManifestResourceStream(@"QLFUI.Res.border.bordermr.bmp")));
            Bitmap b5 = this.ptbtl.BackgroundImage as Bitmap;
            b5.MakeTransparent(Color.FromArgb(255, 0, 255));

            this.panelt.BackgroundImage = new Bitmap(Image.FromStream(Assembly.GetExecutingAssembly().GetManifestResourceStream(@"QLFUI.Res.border.bordertm.bmp")));
            this.panelm.BackgroundImage = new Bitmap(Image.FromStream(Assembly.GetExecutingAssembly().GetManifestResourceStream(@"QLFUI.Res.border.bg.bmp")));
            this.panelb.BackgroundImage = new Bitmap(Image.FromStream(Assembly.GetExecutingAssembly().GetManifestResourceStream(@"QLFUI.Res.border.borderbm.bmp")));

            Bitmap closeBitmap = new Bitmap(Image.FromStream(Assembly.GetExecutingAssembly().GetManifestResourceStream(@"QLFUI.Res.border.CloseBtn.bmp")));
            closeBitmap.MakeTransparent(Color.FromArgb(255, 0, 255));
            this.btclose.NormalImage = closeBitmap.Clone(new Rectangle(0, 0, 37, 22), PixelFormat.Format64bppPArgb);
            this.btclose.MoveImage = closeBitmap.Clone(new Rectangle(37, 0, 37, 22), PixelFormat.Format64bppPArgb);
            this.btclose.DownImage = closeBitmap.Clone(new Rectangle(74, 0, 37, 22), PixelFormat.Format64bppPArgb);
            this.btclose.BackgroundImage = this.btclose.NormalImage;

            Bitmap minBitmap = new Bitmap(Image.FromStream(Assembly.GetExecutingAssembly().GetManifestResourceStream(@"QLFUI.Res.border.MinBtn.bmp")));
            minBitmap.MakeTransparent(Color.FromArgb(255, 0, 255));
            this.btmin.NormalImage = minBitmap.Clone(new Rectangle(0, 0, 31, 22), PixelFormat.Format64bppPArgb);
            this.btmin.MoveImage = minBitmap.Clone(new Rectangle(31, 0, 31, 22), PixelFormat.Format64bppPArgb);
            this.btmin.DownImage = minBitmap.Clone(new Rectangle(62, 0, 31, 22), PixelFormat.Format64bppPArgb);
            this.btmin.BackgroundImage = this.btmin.NormalImage;

            Bitmap maxBitmap = new Bitmap(Image.FromStream(Assembly.GetExecutingAssembly().GetManifestResourceStream(@"QLFUI.Res.border.MaxBtn.bmp")));
            maxBitmap.MakeTransparent(Color.FromArgb(255, 0, 255));
            this.btmax.NormalImage = maxBitmap.Clone(new Rectangle(0, 0, 32, 22), PixelFormat.Format64bppPArgb);
            this.btmax.MoveImage = maxBitmap.Clone(new Rectangle(32, 0, 32, 22), PixelFormat.Format64bppPArgb);
            this.btmax.DownImage = maxBitmap.Clone(new Rectangle(64, 0, 32, 22), PixelFormat.Format64bppPArgb);
            this.btmax.BackgroundImage = this.btmax.NormalImage;

            Bitmap resBitmap = new Bitmap(Image.FromStream(Assembly.GetExecutingAssembly().GetManifestResourceStream(@"QLFUI.Res.button.RestrBtn.bmp")));
            resBitmap.MakeTransparent(Color.FromArgb(255, 0, 255));
            this.btres.NormalImage = resBitmap.Clone(new Rectangle(0, 0, 32, 22), PixelFormat.Format64bppPArgb);
            this.btres.MoveImage = resBitmap.Clone(new Rectangle(32, 0, 32, 22), PixelFormat.Format64bppPArgb);
            this.btres.DownImage = resBitmap.Clone(new Rectangle(64, 0, 32, 22), PixelFormat.Format64bppPArgb);
            this.btres.BackgroundImage = this.btres.NormalImage;

            Win32.SetWindowLong(this.Handle, -16, Win32.WS_SYSMENU + Win32.WS_MINIMIZEBOX);
        }

        #endregion

        #region 事件

        private void panelt_MouseDown(object sender, MouseEventArgs e)
        {
            if (st != stat.Max)
            {
                if (e.Button == MouseButtons.Left)
                {
                    Win32.ReleaseCapture();
                    Win32.SendMessage(Handle, 274, 61440 + 9, 0);
                }
            }

        }


        private void btclose_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void btmin_Click(object sender, EventArgs e)
        {
            this.WindowState = FormWindowState.Minimized;
        }

        private void btmax_Click(object sender, EventArgs e)
        {
            this.btmax.Top -= 30;
            this.btres.Top = 0;

            //最大化之前记录窗口信息便于缩小
            orginHeight = this.Height;
            orginWidth = this.Width;
            X = this.Location.X;
            Y = this.Location.Y;

            this.Top = -1;
            this.Left = -1;
            BorderWidth = Screen.PrimaryScreen.WorkingArea.Width + 4;
            BorderHeight = Screen.PrimaryScreen.WorkingArea.Height + 4;

            st = stat.Max;
        }

        private void btres_Click(object sender, EventArgs e)
        {
            this.btmax.Top = 0;
            this.btres.Top -= 30;

            this.Left = X;
            this.Top = Y;
            BorderWidth = orginWidth;
            BorderHeight = orginHeight;
            st = stat.Normal;
        }

        #endregion

        protected override void OnResize(EventArgs e)
        {
            base.OnResize(e);

            int Rgn = Win32.CreateRoundRectRgn(1, 1, this.Width - 1, this.Height - 1, 7, 7);
            Win32.SetWindowRgn(this.Handle, Rgn, true);
        }



    }
}
